"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DelayedOnChange = void 0;
var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));
var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));
var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));
var _react = _interopRequireWildcard(require("react"));
var _isEqual = _interopRequireDefault(require("lodash/isEqual"));
var _excluded = ["children"];
var emptyFunction = function emptyFunction() {
  return undefined;
};

/**
 * This component is used to wrap Input and Textarea components to optimize form re-render.
 * These 2 are the only components that trigger form model change on each character input.
 * This means, whenever you type a letter an entire form re-renders.
 * On complex forms you will feel and see a significant delay if this component is not used.
 *
 * The logic behind this component is to serve as a middleware between Form and Input/Textarea, and only notify form of a change when
 * a user stops typing for given period of time (400ms by default).
 */

var DelayedOnChange = exports.DelayedOnChange = function DelayedOnChange(_ref) {
  var children = _ref.children,
    other = (0, _objectWithoutProperties2.default)(_ref, _excluded);
  var firstMount = (0, _react.useRef)(true);
  var onChange = other.onChange,
    _other$delay = other.delay,
    delay = _other$delay === void 0 ? 400 : _other$delay,
    initialValue = other.value;
  var _useState = (0, _react.useState)(initialValue),
    _useState2 = (0, _slicedToArray2.default)(_useState, 2),
    value = _useState2[0],
    setValue = _useState2[1];
  // Sync state and props
  (0, _react.useEffect)(function () {
    // Do not update local state, if the incoming value is the same as the local state.
    // This is primarily an optimization for non-scalar values (objects).
    if ((0, _isEqual.default)(initialValue, value)) {
      return;
    }
    setValue(initialValue);
  }, [initialValue]);
  var localTimeout = _react.default.useRef(null);
  var applyValue = function applyValue(value) {
    localTimeout.current && clearTimeout(localTimeout.current);
    localTimeout.current = null;
    if (!onChange) {
      return;
    }
    onChange(value);
  };
  var onChangeLocal = _react.default.useCallback(function (value) {
    setValue(value);
  }, []);

  // this is fired upon change value state
  var onValueStateChanged = function onValueStateChanged(nextValue) {
    // We don't want to execute callbacks, if the value hasn't changed.
    if ((0, _isEqual.default)(nextValue, initialValue)) {
      return;
    }
    localTimeout.current && clearTimeout(localTimeout.current);
    localTimeout.current = null;
    localTimeout.current = setTimeout(function () {
      return applyValue(nextValue);
    }, delay);
  };

  // need to clear the timeout when unmounting the component
  (0, _react.useEffect)(function () {
    return function () {
      if (!localTimeout.current) {
        return;
      }
      clearTimeout(localTimeout.current);
      localTimeout.current = null;
    };
  }, []);
  (0, _react.useEffect)(function () {
    if (firstMount.current) {
      firstMount.current = false;
      return;
    }
    onValueStateChanged(value);
  }, [value]);
  var newProps = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, other), {}, {
    value: value,
    onChange: onChangeLocal
  });
  var renderProp = typeof children === "function" ? children : null;
  var child = renderProp ? renderProp(newProps) : /*#__PURE__*/_react.default.cloneElement(children, newProps);
  var props = (0, _objectSpread2.default)({}, child.props);
  var realOnKeyDown = props.onKeyDown || emptyFunction;
  var realOnBlur = props.onBlur || emptyFunction;

  // Need to apply value if input lost focus
  var onBlur = function onBlur(ev) {
    if (!ev["persist"]) {
      return;
    }
    ev.persist();
    applyValue(ev.target.value);
    realOnBlur(ev);
  };

  // Need to listen for TAB key to apply new value immediately, without delay. Otherwise validation will be triggered with old value.
  var onKeyDown = function onKeyDown(ev) {
    ev.persist();
    if (ev.key === "Tab") {
      applyValue(ev.target.value);
      realOnKeyDown(ev);
    } else if (ev.key === "Enter") {
      applyValue(ev.target.value);
      realOnKeyDown(ev);
    } else {
      realOnKeyDown(ev);
    }
  };
  return /*#__PURE__*/_react.default.cloneElement(child, (0, _objectSpread2.default)((0, _objectSpread2.default)({}, props), {}, {
    onBlur: onBlur,
    onKeyDown: onKeyDown
  }));
};

//# sourceMappingURL=DelayedOnChange.js.map